@@ -1,11 +1,11 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2018 - 2021 PyTroll
+# Copyright (c) 2018 - 2022 Pytroll Developers
 
 # Author(s):
 
-#   Adam.Dybbroe <adam.dybbroe@smhi.se>
+#   Adam.Dybbroe <Firstname.Lastname at smhi.se>
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,18 +25,20 @@
 
 import threading
 from trollsift.parser import parse  # @UnresolvedImport
+from trollsift.parser import globify
+# from trollsift import Parser
 from posttroll.message import Message  # @UnresolvedImport
 from subprocess import Popen, PIPE
 import os
 import stat
-import netifaces
 import shlex
 from glob import glob
 import socket
 from datetime import datetime, timedelta
 #: Python 2/3 differences
-from six.moves.urllib.parse import urlparse  # @UnresolvedImport
+from six.moves.urllib.parse import urlparse
 
+from posttroll.address_receiver import get_local_ips
 
 import logging
 LOG = logging.getLogger(__name__)
@@ -46,6 +48,10 @@ class NwpPrepareError(Exception):
     pass
 
 
+class FindTimeControlFileError(Exception):
+    pass
+
+
 PPS_OUT_PATTERN = ("S_NWC_{segment}_{orig_platform_name}_{orbit_number:05d}_" +
                    "{start_time:%Y%m%dT%H%M%S%f}Z_{end_time:%Y%m%dT%H%M%S%f}Z.{extention}")
 PPS_OUT_PATTERN_MULTIPLE = ("S_NWC_{segment1}_{segment2}_{orig_platform_name}_{orbit_number:05d}_" +
@@ -53,17 +59,17 @@ PPS_OUT_PATTERN_MULTIPLE = ("S_NWC_{segment1}_{segment2}_{orig_platform_name}_{o
 PPS_STAT_PATTERN = ("S_NWC_{segment}_{orig_platform_name}_{orbit_number:05d}_" +
                     "{start_time:%Y%m%dT%H%M%S%f}Z_{end_time:%Y%m%dT%H%M%S%f}Z_statistics.xml")
 
-SUPPORTED_NOAA_SATELLITES = ['NOAA-15', 'NOAA-18', 'NOAA-19']
-SUPPORTED_METOP_SATELLITES = ['Metop-B', 'Metop-A', 'Metop-C']
-SUPPORTED_EOS_SATELLITES = ['EOS-Terra', 'EOS-Aqua']
-SUPPORTED_JPSS_SATELLITES = ['Suomi-NPP', 'NOAA-20', 'NOAA-21']
-SUPPORTED_METEOSAT_SATELLITES = ['Meteosat-09', 'Meteosat-10', 'Meteosat-11']
+SUPPORTED_AVHRR_SATELLITES = ['NOAA-15', 'NOAA-18', 'NOAA-19',
+                              'Metop-B', 'Metop-A', 'Metop-C']
+SUPPORTED_EARS_AVHRR_SATELLITES = ['Metop-B', 'Metop-C']
+SUPPORTED_MODIS_SATELLITES = ['EOS-Terra', 'EOS-Aqua']
+SUPPORTED_VIIRS_SATELLITES = ['Suomi-NPP', 'NOAA-20', 'NOAA-21']
+SUPPORTED_SEVIRI_SATELLITES = ['Meteosat-09', 'Meteosat-10', 'Meteosat-11']
 
-SUPPORTED_PPS_SATELLITES = (SUPPORTED_NOAA_SATELLITES +
-                            SUPPORTED_METOP_SATELLITES +
-                            SUPPORTED_EOS_SATELLITES +
-                            SUPPORTED_METEOSAT_SATELLITES +
-                            SUPPORTED_JPSS_SATELLITES)
+SUPPORTED_PPS_SATELLITES = (SUPPORTED_AVHRR_SATELLITES +
+                            SUPPORTED_MODIS_SATELLITES +
+                            SUPPORTED_SEVIRI_SATELLITES +
+                            SUPPORTED_VIIRS_SATELLITES)
 
 GEOLOC_PREFIX = {'EOS-Aqua': 'MYD03', 'EOS-Terra': 'MOD03'}
 DATA1KM_PREFIX = {'EOS-Aqua': 'MYD021km', 'EOS-Terra': 'MOD021km'}
@@ -198,17 +204,6 @@ def message_uid(msg):
     return SceneId(platform_name, orbit_number, starttime)
 
 
-def get_local_ips():
-    inet_addrs = [netifaces.ifaddresses(iface).get(netifaces.AF_INET)
-                  for iface in netifaces.interfaces()]
-    ips = []
-    for addr in inet_addrs:
-        if addr is not None:
-            for add in addr:
-                ips.append(add['addr'])
-    return ips
-
-
 def get_sceneid(platform_name, orbit_number, starttime):
 
     if starttime:
@@ -238,7 +233,7 @@ def ready2run(msg, files4pps, **kwargs):
 
     uris = []
     if (msg.type == 'dataset' and
-            msg.data['platform_name'] in SUPPORTED_EOS_SATELLITES):
+            msg.data['platform_name'] in SUPPORTED_MODIS_SATELLITES):
         LOG.info('Dataset: ' + str(msg.data['dataset']))
         LOG.info('Got a dataset on an EOS satellite')
         LOG.info('\t ...thus we can assume we have everything we need for PPS')
@@ -246,7 +241,7 @@ def ready2run(msg, files4pps, **kwargs):
             uris.append(obj['uri'])
 
     elif (sdr_granule_processing and msg.type == 'dataset' and
-            msg.data['platform_name'] in SUPPORTED_JPSS_SATELLITES):
+            msg.data['platform_name'] in SUPPORTED_VIIRS_SATELLITES):
         LOG.info('Dataset: ' + str(msg.data['dataset']))
         LOG.info('Got a dataset on a JPSS/SNPP satellite')
         if destination is None:
@@ -294,19 +289,19 @@ def ready2run(msg, files4pps, **kwargs):
                  "Continue...")
         return False
 
-    if msg.data['platform_name'] in SUPPORTED_METEOSAT_SATELLITES:
+    if msg.data['platform_name'] in SUPPORTED_SEVIRI_SATELLITES:
         if msg.data['sensor'] not in ['seviri', ]:
             LOG.info(
                 'Sensor ' + str(msg.data['sensor']) +
                 ' not required for MODIS PPS processing...')
             return False
-    elif msg.data['platform_name'] in SUPPORTED_EOS_SATELLITES:
+    elif msg.data['platform_name'] in SUPPORTED_MODIS_SATELLITES:
         if msg.data['sensor'] not in ['modis', ]:
             LOG.info(
                 'Sensor ' + str(msg.data['sensor']) +
                 ' not required for MODIS PPS processing...')
             return False
-    elif msg.data['platform_name'] in SUPPORTED_JPSS_SATELLITES:
+    elif msg.data['platform_name'] in SUPPORTED_VIIRS_SATELLITES:
         if msg.data['sensor'] not in ['viirs', ]:
             LOG.info(
                 'Sensor ' + str(msg.data['sensor']) +
@@ -347,7 +342,7 @@ def ready2run(msg, files4pps, **kwargs):
         files4pps[sceneid] = []
 
     LOG.debug("level1_files = %s", level1_files)
-    if platform_name in SUPPORTED_EOS_SATELLITES:
+    if platform_name in SUPPORTED_MODIS_SATELLITES:
         for item in level1_files:
             fname = os.path.basename(item)
             LOG.debug("EOS level-1 file: %s", item)
@@ -361,14 +356,13 @@ def ready2run(msg, files4pps, **kwargs):
 
     LOG.debug("files4pps: %s", str(files4pps[sceneid]))
     if (stream_tag_name in msg.data and msg.data[stream_tag_name] in [stream_name, ] and
-            platform_name in SUPPORTED_METOP_SATELLITES):
+            platform_name in SUPPORTED_EARS_AVHRR_SATELLITES):
         LOG.info("EARS Metop data. Only require the HRPT/AVHRR level-1b file to be ready!")
-    elif (platform_name in SUPPORTED_METOP_SATELLITES or
-          platform_name in SUPPORTED_NOAA_SATELLITES):
+    elif (platform_name in SUPPORTED_AVHRR_SATELLITES):
         if len(files4pps[sceneid]) < len(REQUIRED_MW_SENSORS[platform_name]) + 1:
             LOG.info("Not enough NOAA/Metop sensor data available yet...")
             return False
-    elif platform_name in SUPPORTED_EOS_SATELLITES:
+    elif platform_name in SUPPORTED_MODIS_SATELLITES:
         if len(files4pps[sceneid]) < 2:
             LOG.info("Not enough MODIS level 1 files available yet...")
             return False
@@ -406,29 +400,30 @@ def prepare_pps_arguments(platform_name, level1_filepath, **kwargs):
     orbit_number = kwargs.get('orbit_number')
     pps_args = {}
 
-    if platform_name in SUPPORTED_EOS_SATELLITES:
+    if platform_name in SUPPORTED_MODIS_SATELLITES:
         pps_args['modisorbit'] = orbit_number
         pps_args['modisfile'] = level1_filepath
 
-    elif platform_name in SUPPORTED_JPSS_SATELLITES:
+    elif platform_name in SUPPORTED_VIIRS_SATELLITES:
         pps_args['csppfile'] = level1_filepath
 
-    elif platform_name in SUPPORTED_METOP_SATELLITES:
-        pps_args['hrptfile'] = level1_filepath
-
-    elif platform_name in SUPPORTED_NOAA_SATELLITES:
+    elif platform_name in SUPPORTED_AVHRR_SATELLITES:
         pps_args['hrptfile'] = level1_filepath
 
     return pps_args
 
 
 def create_pps_call_command_sequence(pps_script_name, scene, options):
+    """Create the PPS call commnd sequence.
+
+    Applies to NWCSAF/PPS v2014.
+    """
     LVL1_NPP_PATH = os.environ.get('LVL1_NPP_PATH',
                                    options.get('LVL1_NPP_PATH', None))
     LVL1_EOS_PATH = os.environ.get('LVL1_EOS_PATH',
                                    options.get('LVL1_EOS_PATH', None))
 
-    if scene['platform_name'] in SUPPORTED_EOS_SATELLITES:
+    if scene['platform_name'] in SUPPORTED_MODIS_SATELLITES:
         cmdstr = "%s %s %s %s %s" % (pps_script_name,
                                      SATELLITE_NAME[
                                          scene['platform_name']],
@@ -443,30 +438,36 @@ def create_pps_call_command_sequence(pps_script_name, scene, options):
 
     cmdstr = cmdstr + ' ' + str(options['aapp_level1files_max_minutes_old'])
 
-    if scene['platform_name'] in SUPPORTED_JPSS_SATELLITES and LVL1_NPP_PATH:
+    if scene['platform_name'] in SUPPORTED_VIIRS_SATELLITES and LVL1_NPP_PATH:
         cmdstr = cmdstr + ' ' + str(LVL1_NPP_PATH)
-    elif scene['platform_name'] in SUPPORTED_EOS_SATELLITES and LVL1_EOS_PATH:
+    elif scene['platform_name'] in SUPPORTED_MODIS_SATELLITES and LVL1_EOS_PATH:
         cmdstr = cmdstr + ' ' + str(LVL1_EOS_PATH)
 
     return shlex.split(str(cmdstr))
 
 
-def create_pps2018_call_command(python_exec, pps_script_name, scene, sequence=True):
+def create_pps_call_command(python_exec, pps_script_name, scene, use_l1c=False):
+    """Create the pps call command.
 
-    if scene['platform_name'] in SUPPORTED_EOS_SATELLITES:
-        cmdstr = ("%s " % python_exec + " %s " % pps_script_name +
-                  " --modisfile %s" % scene['file4pps'])
-    elif scene['platform_name'] in SUPPORTED_JPSS_SATELLITES:
-        cmdstr = ("%s " % python_exec + " %s " % pps_script_name +
-                  " --csppfile %s" % scene['file4pps'])
-    elif scene['platform_name'] in SUPPORTED_METEOSAT_SATELLITES:
+    Supports PPSv2018 and PPSv2021.
+    """
+    if use_l1c:
         cmdstr = ("%s" % python_exec + " %s " % pps_script_name +
                   "-af %s" % scene['file4pps'])
-    else:
+        LOG.debug("PPS call command: %s", str(cmdstr))
+        return cmdstr
+
+    if scene['platform_name'] in SUPPORTED_MODIS_SATELLITES:
+        cmdstr = ("%s " % python_exec + " %s " % pps_script_name +
+                  " --modisfile %s" % scene['file4pps'])
+    elif scene['platform_name'] in SUPPORTED_VIIRS_SATELLITES:
+        cmdstr = ("%s " % python_exec + " %s " % pps_script_name +
+                  " --csppfile %s" % scene['file4pps'])
+    elif scene['platform_name'] in SUPPORTED_AVHRR_SATELLITES:
         cmdstr = ("%s " % python_exec + " %s " % pps_script_name +
                   " --hrptfile %s" % scene['file4pps'])
-    if sequence:
-        return shlex.split(str(cmdstr))
+    else:
+        raise
 
     return cmdstr
 
@@ -476,23 +477,19 @@ def get_pps_inputfile(platform_name, ppsfiles):
     file used in the PPS call
     """
 
-    if platform_name in SUPPORTED_EOS_SATELLITES:
+    if platform_name in SUPPORTED_MODIS_SATELLITES:
         for ppsfile in ppsfiles:
             if os.path.basename(ppsfile).find('021km') > 0:
                 return ppsfile
-    elif platform_name in SUPPORTED_NOAA_SATELLITES:
+    elif platform_name in SUPPORTED_AVHRR_SATELLITES:
         for ppsfile in ppsfiles:
             if os.path.basename(ppsfile).find('hrpt_') >= 0:
                 return ppsfile
-    elif platform_name in SUPPORTED_METOP_SATELLITES:
-        for ppsfile in ppsfiles:
-            if os.path.basename(ppsfile).find('hrpt_') >= 0:
-                return ppsfile
-    elif platform_name in SUPPORTED_JPSS_SATELLITES:
+    elif platform_name in SUPPORTED_VIIRS_SATELLITES:
         for ppsfile in ppsfiles:
             if os.path.basename(ppsfile).find('SVM01') >= 0:
                 return ppsfile
-    elif platform_name in SUPPORTED_METEOSAT_SATELLITES:
+    elif platform_name in SUPPORTED_SEVIRI_SATELLITES:
         for ppsfile in ppsfiles:
             if os.path.basename(ppsfile).find('NWC') >= 0:
                 return ppsfile
@@ -535,8 +532,14 @@ def get_outputfiles(path, platform_name, orb, st_time='', **kwargs):
     if xml_output:
         filelist = filelist + get_xml_outputfiles(path, platform_name, orb, st_time)
 
+    return filter4oldfiles(filelist)
+
+
+def filter4oldfiles(filelist, minutes_thr=90.):
+    """Check if the PPS file is older than threshold and only consider fresh ones."""
+
     now = datetime.utcnow()
-    time_threshold = timedelta(minutes=90.)
+    time_threshold = timedelta(minutes=minutes_thr)
     filtered_flist = []
     for fname in filelist:
         mtime = datetime.utcfromtimestamp(os.stat(fname)[stat.ST_MTIME])
@@ -584,12 +587,149 @@ def get_xml_outputfiles(path, platform_name, orb, st_time=''):
     return filelist
 
 
+def create_xml_timestat_from_lvl1c(scene, pps_control_path):
+    """From lvl1c file create XML file and return a file list."""
+    try:
+        return [create_pps_file_from_lvl1c(scene['file4pps'], pps_control_path, "timectrl", ".xml")]
+    except KeyError:
+        return []
+
+
+def create_product_statistics_from_lvl1c(scene, pps_control_path):
+    """From lvl1c file create product XML files and return a file list."""
+    try:
+        glob_pattern = create_pps_file_from_lvl1c(scene['file4pps'], pps_control_path, "*", "_statistics.xml")
+        return glob(glob_pattern)
+    except KeyError:
+        return []
+
+
+def create_pps_file_from_lvl1c(l1c_file_name, pps_control_path, name_tag, file_type):
+    """From lvl1c file create name_tag-file of type file_type."""
+    from trollsift import parse, compose
+    f_pattern = 'S_NWC_{name_tag}_{platform_id}_{orbit_number}_{start_time}Z_{end_time}Z{file_type}'
+    l1c_path, l1c_file = os.path.split(l1c_file_name)
+    data = parse(f_pattern, l1c_file)
+    data["name_tag"] = name_tag
+    data["file_type"] = file_type
+    return os.path.join(pps_control_path, compose(f_pattern, data))
+
+
+def create_xml_timestat_from_ascii(scene, pps_control_path):
+    """From ascii file(s) with PPS time statistics create XML file(s) and return a file list."""
+    try:
+        from pps_time_control import PPSTimeControl
+    except ImportError:
+        LOG.warning("Failed to import the PPSTimeControl from pps")
+        return []
+
+    try:
+        infile = get_time_control_ascii_filename(scene, pps_control_path)
+    except FindTimeControlFileError:
+        LOG.exception('No XML Time statistics file created!')
+        return []
+
+    LOG.info("Time control ascii file: " + str(infile))
+    LOG.info("Read time control ascii file and generate XML")
+    ppstime_con = PPSTimeControl(infile)
+    ppstime_con.sum_up_processing_times()
+    try:
+        ppstime_con.write_xml()
+    except Exception as e:  # TypeError as e:
+        LOG.warning('Not able to write time control xml file')
+        LOG.warning(e)
+
+    # There should always be only one xml file for each ascii file found above!
+    xmlfile = infile.replace('.txt', '.xml')
+    return filter4oldfiles([xmlfile], 90.)
+
+
+def get_time_control_ascii_filename(scene, pps_control_path):
+    """From the scene object and a file path get the time-control-ascii-filename (with path)."""
+    infiles = get_time_control_ascii_filename_candidates(scene, pps_control_path)
+    LOG.info("Time control ascii file candidates: " + str(infiles))
+    if len(infiles) == 0:
+        raise FindTimeControlFileError("No time control ascii file candidate found!")
+    elif len(infiles) > 1:
+        msg = "More than one time control ascii file candidate found - unresolved ambiguity!"
+        raise FindTimeControlFileError(msg)
+
+    return infiles[0]
+
+
+def get_time_control_ascii_filename_candidates(scene, pps_control_path):
+    """From directory path, sensor and platform name get possible time-control filenames."""
+    sensors = SENSOR_LIST.get(scene['platform_name'], scene['platform_name'])
+    platform_id = SATELLITE_NAME.get(scene['platform_name'], scene['platform_name'])
+    LOG.info("pps platform_id = %s", str(platform_id))
+
+    #: Create the start time (format dateTtime) to be used in file findings
+    st_times = []
+    if sensors == 'seviri':
+        st_times.append(scene['starttime'].strftime("%Y%m%dT%H%M%S.%f"))
+    elif sensors in ['viirs', ]:
+        st_times.append(scene['starttime'].strftime("%Y%m%dT%H%M%S"))
+    elif 'avhrr/3' in sensors or sensors in ['modis', ]:
+        # At least for the AVHRR data the PPS filenames differ by a few
+        # seconds from the start time in the message - it seems sufficient
+        # to truncate the seconds, but we check for minutes +- 1
+        atime = scene['starttime'] - timedelta(seconds=60)
+        while atime < scene['starttime'] + timedelta(seconds=120):
+            st_times.append(atime.strftime("%Y%m%dT%H%M"))
+            atime = atime + timedelta(seconds=60)
+
+    # For VIIRS we often see a orbit number difference of 1:
+    norbit_candidates = [scene['orbit_number']]
+    for idx in [1, -1]:
+        norbit_candidates.append(int(scene['orbit_number']) + idx)
+
+    # PPSv2018 MODIS files have the orbit number set to "00000"!
+    # Level1c4pps files have the orbit number configurable with default "00000"!
+    norbit_candidates.append(0)
+
+    infiles = []
+    for norbit in norbit_candidates:
+        for st_time in st_times:
+            txt_time_file = (os.path.join(pps_control_path, 'S_NWC_timectrl_') +
+                             str(METOP_NAME_LETTER.get(platform_id, platform_id)) +
+                             '_' + '%.5d' % norbit + '_' +
+                             st_time +
+                             '*.txt')
+            LOG.info("glob string = " + str(txt_time_file))
+            infiles = infiles + glob(txt_time_file)
+
+    return infiles
+
+
+def get_product_statistics_files(pps_control_path, scene, product_statistics_filename,
+                                 max_abs_deviation_minutes):
+    """From directory path, sensor and platform name get possible product statistics filenames."""
+
+    platform_id = SATELLITE_NAME.get(scene['platform_name'], scene['platform_name'])
+    platform_id = METOP_NAME_LETTER.get(platform_id, platform_id)
+    product_stat_flist = []
+    scene_start_time = scene['starttime']
+    possible_filetimes = [scene_start_time]
+    for nmin in range(1, max_abs_deviation_minutes + 1):
+        possible_filetimes.append(scene_start_time - timedelta(seconds=60 * nmin))
+        possible_filetimes.append(scene_start_time + timedelta(seconds=60 * nmin))
+
+    for product_name in ['CMA', 'CT', 'CTTH', 'CPP', 'CMAPROB']:
+        for start_time in possible_filetimes:
+            glbify = globify(product_statistics_filename, {'product': product_name,
+                                                           'satellite': platform_id,
+                                                           'orbit': '%.5d' % scene['orbit_number'],
+                                                           'starttime': start_time})
+            product_stat_flist = product_stat_flist + glob(os.path.join(pps_control_path, glbify))
+
+    return product_stat_flist
+
+
 def publish_pps_files(input_msg, publish_q, scene, result_files, **kwargs):
     """
     Publish messages for the files provided.
     """
 
-    environment = kwargs.get('environment')
     servername = kwargs.get('servername')
     station = kwargs.get('station', 'unknown')
 
@@ -597,7 +737,7 @@ def publish_pps_files(input_msg, publish_q, scene, result_files, **kwargs):
         # Get true start and end time from filenames and adjust the end time in
         # the publish message:
         filename = os.path.basename(result_file)
-        LOG.info("file to publish = " + str(filename))
+        LOG.info("file to publish = %s", str(filename))
         try:
             try:
                 metadata = parse(PPS_OUT_PATTERN, filename)
@@ -638,7 +778,7 @@ def publish_pps_files(input_msg, publish_q, scene, result_files, **kwargs):
         to_send['start_time'], to_send['end_time'] = starttime, endtime
         pubmsg = Message('/' + to_send['format'] + '/' +
                          to_send['data_processing_level'] +
-                         '/' + station + '/' + environment +
+                         '/' + station +
                          '/polar/direct_readout/',
                          "file", to_send).encode()
         LOG.info("Sending: %s", str(pubmsg))
